
/* Programme pour Arduino Max7456 et capteur DS18B20
F5MNA    Site http://f5mna.free.fr
Mesure Pin digital N° 7
        
  __________
  !         !---------------------------------|---- + 5 V
  ! DS18B20 !----------------------|-/\/\/\/--|
  !_________!-------------- Masse  |
                                   |
                                   | Pin D7
  */

#define DATAOUT 11
#define DATAIN  12
#define SPICLOCK  13
#define MAX7456SELECT 10


#define DMM_reg   0x04
#define DMAH_reg  0x05
#define DMAL_reg  0x06
#define DMDI_reg  0x07
#define VM0_reg   0x00
#define VM1_reg   0x01


#define VIDEO_BUFFER_DISABLE 0x01
#define MAX7456_RESET 0x02
#define VERTICAL_SYNC_NEXT_VSYNC 0x04
#define OSD_ENABLE 0x08
#define SYNC_MODE_AUTO 0x00
#define SYNC_MODE_INTERNAL 0x30
#define SYNC_MODE_EXTERNAL 0x20
#define VIDEO_MODE_PAL 0x40
#define VIDEO_MODE_NTSC 0x00


#define BLINK_DUTY_CYCLE_50_50 0x00
#define BLINK_DUTY_CYCLE_33_66 0x01
#define BLINK_DUTY_CYCLE_25_75 0x02
#define BLINK_DUTY_CYCLE_75_25 0x03

#define BLINK_TIME_0 0x00
#define BLINK_TIME_1 0x04
#define BLINK_TIME_2 0x08
#define BLINK_TIME_3 0x0C

#define BACKGROUND_BRIGHTNESS_0 0x00
#define BACKGROUND_BRIGHTNESS_7 0x01
#define BACKGROUND_BRIGHTNESS_14 0x02
#define BACKGROUND_BRIGHTNESS_21 0x03
#define BACKGROUND_BRIGHTNESS_28 0x04
#define BACKGROUND_BRIGHTNESS_35 0x05
#define BACKGROUND_BRIGHTNESS_42 0x06
#define BACKGROUND_BRIGHTNESS_49 0x07

#define BACKGROUND_MODE_GRAY 0x40

#define CLEAR_display 0x04
#define CLEAR_display_vert 0x06
#define END_string 0xff

#define ENABLE_display 0x48
#define ENABLE_display_vert 0x4c
#define MAX7456_reset 0x42
#define DISABLE_display 0x40


#define WHITE_level_80 0x03
#define WHITE_level_90 0x02
#define WHITE_level_100 0x01
#define WHITE_level_120 0x00

#define MAX_screen_size 480
#define MAX_screen_rows 16


//  Concerne les déclarations pour le programme DS18B20
  #include <OneWire.h>
  OneWire  ds(7);       
  byte i;
  byte present = 0;
  byte data[12];
  byte addr[8];
  int HighByte, LowByte, SignBit, tempc, Fract, Tlecture, Tc_100; 
 
//////////////////////////////////////////////////////////////
void setup()
{
  byte spi_junk, eeprom_junk;
  int x;
  Serial.begin(9600);
  Serial.flush();

  pinMode(MAX7456SELECT,OUTPUT);
  digitalWrite(MAX7456SELECT,HIGH); 

  pinMode(DATAOUT, OUTPUT);
  pinMode(DATAIN, INPUT);
  pinMode(SPICLOCK,OUTPUT);
  //pinMode(VSYNC, INPUT);

  SPCR = (1<<SPE)|(1<<MSTR);
  spi_junk=SPSR;
  spi_junk=SPDR;
  delay(250);

  digitalWrite(MAX7456SELECT,LOW);
  spi_transfer(VM0_reg);
  spi_transfer(MAX7456_reset);
  digitalWrite(MAX7456SELECT,HIGH);
  delay(500);

  digitalWrite(MAX7456SELECT,LOW);
  for (x = 0; x < MAX_screen_rows; x++)
  {
    spi_transfer(x + 0x10);
    spi_transfer(WHITE_level_90);
  }

  spi_transfer(VM0_reg);
  spi_transfer(VERTICAL_SYNC_NEXT_VSYNC|OSD_ENABLE);
  digitalWrite(MAX7456SELECT,HIGH);
  delay(100);

  spi_transfer(VM1_reg);
  spi_transfer(BLINK_DUTY_CYCLE_50_50|BLINK_TIME_3);
  digitalWrite(MAX7456SELECT,HIGH);
  
  OSD_write_to_screen("http://f5mna.free.fr",4,4,0,1);
  
}
//////////////////////////////////////////////////////////////
///                Mesure température
/////////////////////////////////////////////////////////////
void Mesuretemp()
{

    if ( !ds.search(addr)) { //Recherche de l'adresse du capteur
      delay(1000);
      ds.reset_search();
      return;
  }

  if ( OneWire::crc8( addr, 7) != addr[7]) {
      delay(1000);
      return;
  }

  int foo, bar;

  ds.reset();
  ds.select(addr);
  ds.write(0x44,1);
  
  present = ds.reset();
  ds.select(addr);    
  ds.write(0xBE);

  for ( i = 0; i < 9; i++) {
    data[i] = ds.read();
  }  
  LowByte = data[0];
  HighByte = data[1];
  Tlecture = (HighByte << 8) + LowByte;
  SignBit = Tlecture & 0x8000;  
  
  if (SignBit) {
    Tlecture = -Tlecture;
  }
  Tc_100 = (6 * Tlecture) + Tlecture / 4;
  tempc = Tc_100 / 100;        
  Fract = Tc_100 % 100;
    if (Fract > 49) {
    if (SignBit) {
      --tempc;
    } else {
      ++tempc;
    }
  }

  if (SignBit) {  
     OSD_write_to_screen("-",6,2,0,1);
  }else{
     OSD_write_to_screen("+",7,2,0,1);
    }
  if (tempc == 0){
  OSD_write_to_screen(" ",6,2,0,1);
  } 
  // passage integer à string
  char temperature[10]; 
 
  itoa(tempc, temperature, 10); 
         if (SignBit) {  
         OSD_write_to_screen("-",7,2,0,1);
         }else{
         OSD_write_to_screen("+",7,2,0,1);
         }
  if (tempc == 0){
          OSD_write_to_screen(" ",7,2,0,1);
  } 
         
  if ((tempc == 0)or (tempc == 1)) {
  OSD_write_to_screen(temperature,9,2,0,1);  
  OSD_write_to_screen(" Degre C   ",10,2,0,1);
  }  
 
  if (tempc > 9){
  OSD_write_to_screen(temperature,9,2,0,1);
  OSD_write_to_screen(" Degres C",11,2,0,1);
  }
  else{
  OSD_write_to_screen(temperature,9,2,0,1);
  OSD_write_to_screen(" Degres C  ",10,2,0,1);  
  }
  if ((tempc == 0)or (tempc == 1)) {
  OSD_write_to_screen(temperature,9,2,0,1);  
  OSD_write_to_screen(" Degre C   ",10,2,0,1);
  }
 
delay(100); // 
}
//////////////////////////////////////////////////////////////
void Max7456 (){
//////////////////////////////////////////////////////////////

spi_transfer(VM0_reg);
spi_transfer(VERTICAL_SYNC_NEXT_VSYNC|OSD_ENABLE|SYNC_MODE_AUTO);
digitalWrite(MAX7456SELECT,HIGH); 
}
byte convert_ascii (int character)
{
// Transposition de la table ASCII (non standard) integrée à l'EEPROM du MAX7456

 byte lookup_char;

  if (character == 32)
    lookup_char = 0x00; // blank space
  else if (character == 48)
    lookup_char = 0x0a; // 0
  else if ((character > 48) && (character < 58))
    lookup_char = (character - 48); // 1-9
  else if ((character > 64) && (character < 91))
    lookup_char = (character - 54); // A-Z
  else if ((character > 96) && (character < 123))
    lookup_char = (character - 60); // a-z
  else if (character == 34)
    lookup_char = 0x48; // "
  else if (character == 39)
    lookup_char = 0x46; // '
  else if (character == 40)
    lookup_char = 0x3f; // (
  else if (character == 41)
    lookup_char = 0x40; // )
  else if (character == 44)
    lookup_char = 0x45; // ,
  else if (character == 45)
    lookup_char = 0x49; // -
  else if (character == 46)
    lookup_char = 0x41; // .
  else if (character == 47)
    lookup_char = 0x47; // /
  else if (character == 58)
    lookup_char = 0x44; // :
  else if (character == 59)
    lookup_char = 0x43; // ;
  else if (character == 60)
    lookup_char = 0x4a; // <
  else if (character == 62)
    lookup_char = 0x4b; // >
  else if (character == 63)
    lookup_char = 0x42; // ?
  else if (character == 64)
    lookup_char = 0x4c; // @
  else
    lookup_char = 0x00; // out of range, blank space

 return (lookup_char);

}
//////////////////////////////////////////////////////////////
byte spi_transfer(volatile byte data)
{
  SPDR = data;			
  while (!(SPSR & (1<<SPIF)))  
  {
  };
  return SPDR;			
}

void OSD_write_to_screen(const char s[], byte x, byte y, byte blink, byte invert){
  unsigned int linepos;
  byte local_count;
  byte settings, char_address_hi, char_address_lo;
  byte screen_char;
  
  local_count = 0;
  
  char_address_hi = 0;
  char_address_lo = 0;
  
  linepos = y*30+x;

  char_address_hi = linepos >> 8;
  char_address_lo = linepos;

  settings = B00000001;

  if (blink) {
    settings |= (1 << 4);  }

  if (invert){
    settings |= (1 << 3);	  }


  digitalWrite(MAX7456SELECT,LOW);

  spi_transfer(DMM_reg); //dmm
  spi_transfer(settings);

  spi_transfer(DMAH_reg); 
  spi_transfer(char_address_hi);

  spi_transfer(DMAL_reg);
  spi_transfer(char_address_lo);

  while(s[local_count]!='\0')
  {
    screen_char = s[local_count];
    screen_char= convert_ascii(screen_char);
    spi_transfer(DMDI_reg);
    spi_transfer(screen_char);
    local_count++;
  }  
  spi_transfer(DMDI_reg);
  spi_transfer(END_string);
  spi_transfer(VM0_reg); 
  spi_transfer(ENABLE_display_vert);
  digitalWrite(MAX7456SELECT,HIGH);
  }
    
  void loop (){
  Mesuretemp();
  Max7456();
  }
